import argparse
import sys
import makinas
import tensorflow as tf

execname = 'makinas'

def pull():
    parser = argparse.ArgumentParser(description='Pull a model from Makinas',
        usage=execname + ''' pull <runtime> <id>
        ''')
    parser.add_argument('runtime', help="Target Runtime")
    parser.add_argument('id', help="Model ID")
    args = parser.parse_args(sys.argv[2:])
    cli = makinas.Client()
    fname = cli.get_model(args.id+'/'+args.runtime)
    print('Downloaded model: '+fname)

def push():
    parser = argparse.ArgumentParser(description='Push a model to Makinas',
        usage=execname + ''' push <path> [<id>]
        ''')
    parser.add_argument('path', help="Model path")
    parser.add_argument('id', help="Model ID", default=None, nargs='?')
    args = parser.parse_args(sys.argv[2:])
    cli = makinas.Client()
    model = tf.saved_model.load(args.path)
    id = cli.push_model(model, args.id)
    print("Uploaded model: "+id)

def main():
    cmds = {
        'push': push,
        'pull': pull
    }
    parser = argparse.ArgumentParser(description='Makinas CLI Tool',
        usage=execname + ''' <command> [<args>]

Command list:
    push        Push trained model to Makinas
    pull        Pull a model from Makinas
''')
    parser.add_argument('command', help='Subcommand to execute', choices=cmds.keys())

    args = parser.parse_args(sys.argv[1:2])
    if args.command not in cmds:
        print('Unknown command')
        parser.print_help()
        exit(1)
    else:
        cmds[args.command]()



import requests
import os
import zipfile
import tensorflow as tf
import random
import string
import shutil
from tqdm import tqdm

API_DOMAIN = 'https://api.makinas.io'

class Client:

    def __init__(self, api_domain=API_DOMAIN,
                 makinas_key_id=None, makinas_secret=None,
                 makinas_session_token=None):
        self.domain = api_domain


    @staticmethod
    def _download_file(url, progress=False):
        chunksize = 8192
        local_filename = url.split('/')[-1]
        with requests.get(url, stream=True) as r:
            r.raise_for_status()
            len = int(r.headers.get('content-length', 0))
            pbar = tqdm(total=len, unit='iB', unit_scale=True)
            with open(local_filename, 'wb') as f:
                for chunk in r.iter_content(chunk_size=chunksize):
                    if chunk:
                        f.write(chunk)
                        pbar.update(chunksize)
            pbar.close()
        return local_filename


    def get_model(self, model_id):
        return self._download_file(self.domain + '/model/' + model_id)


    def push_model(self, model, model_id=None):
        # Save model
        tmpdir = ''.join(random.choices(string.ascii_letters, k=10))
        #tf.logging.set_verbosity(tf.logging.WARN)
        tf.saved_model.save(model, os.path.join(os.getcwd(), tmpdir))
        # Zip model
        zipf = zipfile.ZipFile(tmpdir+'.zip', 'w', zipfile.ZIP_DEFLATED)
        for root, dirs, files in os.walk(tmpdir):
            for file in files:
                zipf.write(os.path.join(root, file))
        zipf.close()
        # Upload model
        url = self.domain+"/model"
        if model_id is not None:
            url += "/"+model_id
        r = requests.post(url)
        assert r.status_code in [200, 201]
        rsp = r.json()
        putUrl = rsp['url']
        if model_id is None:
            model_id = rsp['id']
        with open(tmpdir+'.zip', 'rb') as zf:
            r = requests.put(putUrl, data=zf.read())
        # Cleanup
        shutil.rmtree(tmpdir)
        os.remove(tmpdir+'.zip')
        return model_id



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="makinas",
    version="0.0.3",
    author="Nemanja Nikodijevic",
    author_email="nemanja.nikodijevic@nbt.ag",
    description="Package for interfacing with Makinas APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/makinas-nbt/pymakinas.git",
    packages=setuptools.find_packages(),
    install_requires=["requests>=2.22.0", "tqdm>=4.41.1"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
    entry_points= {
        'console_scripts': ['makinas=makinas.cli:main'],
    },
)
